from django import template

from django_meta_og.models import Content

register = template.Library()


@register.simple_tag()
def django_meta_og_prefix():
    return "\n".join(
        {f"{meta.property.namespace.prefix}: {meta.property.namespace.uri}" for meta in Content.objects.all()}
    )



from django.test import TestCase

from django_meta_og.models import Content, Namespace, Property


class TestNamespace(TestCase):
    def test_str(self):
        ns, _ = Namespace.objects.get_or_create(prefix="og", uri="https://ogp.me/ns#")
        self.assertEqual(str(ns), "og: https://ogp.me/ns#")


class TestProperty(TestCase):
    def setUp(self):
        self.ns, _ = Namespace.objects.get_or_create(prefix="og", uri="https://ogp.me/ns#")

    def test_str(self):
        prop, _ = Property.objects.get_or_create(namespace=self.ns, name="type")
        self.assertEqual(str(prop), "og:type")


class TestContent(TestCase):
    def setUp(self):
        self.ns, _ = Namespace.objects.get_or_create(prefix="og", uri="https://ogp.me/ns#")
        self.prop, _ = Property.objects.get_or_create(namespace=self.ns, name="type")

    def test_str(self):
        content = Content.objects.create(property=self.prop, content="website")
        self.assertEqual(str(content), "og:type website")



from django.contrib import admin
from django.urls import path
from django.views.generic import TemplateView

urlpatterns = [
    path("page-headers/", TemplateView.as_view(template_name="django_meta_og/header_meta.html")),
    path("test_admin/admin/", admin.site.urls),
]



from django.contrib import admin

from .models import Content, Namespace, Property


@admin.register(Namespace)
class NamespaceAdmin(admin.ModelAdmin):
    ordering = ["prefix"]


@admin.register(Property)
class PropertyAdmin(admin.ModelAdmin):
    ordering = ["namespace", "name"]


@admin.register(Content)
class ContentAdmin(admin.ModelAdmin):
    ordering = ["property", "content"]



from django.test import RequestFactory, TestCase

from django_meta_og.context_processors import meta
from django_meta_og.models import Content, Namespace, Property


class MetaTest(TestCase):
    def test(self):
        ns, _ = Namespace.objects.get_or_create(prefix="og", uri="https://ogp.me/ns#")
        prop, _ = Property.objects.get_or_create(namespace=ns, name="type")
        Content.objects.create(property=prop, content="website")
        context = meta(RequestFactory().request())
        self.assertEqual(list(context.keys()), ["django_meta_og"])
        self.assertQuerySetEqual(context["django_meta_og"], Content.objects.all())



from django.test import TestCase, override_settings

from django_meta_og.models import Content, Namespace, Property


@override_settings(ROOT_URLCONF="django_meta_og.tests.urls")
class TemplateViewTest(TestCase):
    def test(self):
        ns, _ = Namespace.objects.get_or_create(prefix="og", uri="https://ogp.me/ns#")
        prop_type, _ = Property.objects.get_or_create(namespace=ns, name="type")
        prop_title, _ = Property.objects.get_or_create(namespace=ns, name="title")
        Content.objects.create(property=prop_type, content="website")
        Content.objects.create(property=prop_title, content="The Title")

        response = self.client.get("/page-headers/")
        self.assertContains(
            response,
            """
            <meta property="og:type" content="website">
            <meta property="og:title" content="The Title">""",
            html=True,
        )



from django.db import migrations


def noop(apps, schema_editor):
    pass


def create_properties(apps, schema_editor):
    Namespace = apps.get_model('django_meta_og', 'Namespace')
    Property = apps.get_model('django_meta_og', 'Property')

    og, _ = Namespace.objects.get_or_create(prefix="og", defaults={"uri": "https://ogp.me/ns#"})
    for name in (
        # Basic
        "title",
        "type",
        "image",
        "url",
        # Optional
        "audio",
        "description",
        "determiner",
        "locale",
        "locale:alternate",
        "site_name",
        "video",
        # Image
        "image:url",
        "image:secure_url",
        "image:type",
        "image:width",
        "image:height",
        "image:alt",
        # Video
        "video",
        "video:secure_url",
        "video:type",
        "video:width",
        "video:height",
        # Audio
        "audio",
        "audio:secure_url",
        "audio:type",
    ):
        Property.objects.get_or_create(namespace=og, name=name)

    music, _ = Namespace.objects.get_or_create(prefix="music", defaults={"uri": "https://ogp.me/ns/music#"})
    for name in (
        "duration",
        "album",
        "album:disc",
        "album:track",
        "musician",
        "release_date",
        "creator",
    ):
        Property.objects.get_or_create(namespace=music, name=name)

    video, _ = Namespace.objects.get_or_create(prefix="video", defaults={"uri": "https://ogp.me/ns/video#"})
    for name in (
        "actor",
        "actor:role",
        "director",
        "writer",
        "duration",
        "release_date",
        "tag",
        "series",
    ):
        Property.objects.get_or_create(namespace=video, name=name)

    article, _ = Namespace.objects.get_or_create(prefix="article", defaults={"uri": "https://ogp.me/ns/article#"})
    for name in (
        "published_time",
        "modified_time",
        "expiration_time",
        "author",
        "section",
        "tag",
        "isbn",
        "release_date",
    ):
        Property.objects.get_or_create(namespace=article, name=name)

    profile, _ = Namespace.objects.get_or_create(prefix="profile", defaults={"uri": "https://ogp.me/ns/profile#"})
    for name in (
        "first_name",
        "last_name",
        "username",
        "gender",
    ):
        Property.objects.get_or_create(namespace=profile, name=name)


class Migration(migrations.Migration):

    dependencies = [
        ('django_meta_og', '0001_initial'),
    ]
    operations = [
        migrations.RunPython(create_properties, noop),
    ]



# Generated by Django 4.2.6 on 2024-12-10 14:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Namespace',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('prefix', models.CharField(max_length=255, unique=True)),
                ('uri', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Property',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('namespace', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_meta_og.namespace')),
            ],
            options={
                'verbose_name_plural': 'Properties',
                'unique_together': {('namespace', 'name')},
            },
        ),
        migrations.CreateModel(
            name='Content',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.CharField(max_length=255)),
                ('property', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_meta_og.property')),
            ],
        ),
    ]



from django.db import models

from .property import Property


class Content(models.Model):
    property = models.ForeignKey(Property, on_delete=models.CASCADE)
    content = models.CharField(max_length=255)

    def __str__(self):
        return f"{self.property} {self.content}"



SECRET_KEY = "secret"

INSTALLED_APPS = (
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django_meta_og",
)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "tests",
    }
}

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

LANGUAGES = (("en", "English"),)

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django_meta_og.context_processors.meta",
            ],
            "loaders": ["django.template.loaders.app_directories.Loader"],
        },
    }
]



from django.db import models

from .namespace import Namespace


class Property(models.Model):
    namespace = models.ForeignKey(Namespace, on_delete=models.CASCADE)
    name = models.CharField(max_length=255)

    class Meta:
        verbose_name_plural = "Properties"
        unique_together = [["namespace", "name"]]

    def __str__(self):
        return f"{self.namespace.prefix}:{self.name}"



from django.db import models


class Namespace(models.Model):
    prefix = models.CharField(max_length=255, unique=True)
    uri = models.CharField(max_length=255)

    def __str__(self):
        return f"{self.prefix}: {self.uri}"



from django.test import TestCase

from django_meta_og.models import Content, Namespace, Property
from django_meta_og.templatetags.django_meta_og import django_meta_og_prefix


class DjangoMetaOGPrefixTest(TestCase):
    def test(self):
        ns, _ = Namespace.objects.get_or_create(prefix="og", uri="https://ogp.me/ns#")
        prop, _ = Property.objects.get_or_create(namespace=ns, name="type")
        Content.objects.create(property=prop, content="website")
        self.assertEqual(django_meta_og_prefix(), "og: https://ogp.me/ns#")



from unittest import skipIf

import django
from django.contrib import admin
from django.test import TestCase

from django_meta_og.models import Content, Namespace, Property


@skipIf(django.VERSION < (5, 0), "Skip Django < 5.0.")
class AdminTest(TestCase):
    def test_content(self):
        self.assertEqual(admin.site.get_model_admin(Content).ordering, ["property", "content"])

    def test_namespace(self):
        self.assertEqual(admin.site.get_model_admin(Namespace).ordering, ["prefix"])

    def test_property(self):
        self.assertEqual(admin.site.get_model_admin(Property).ordering, ["namespace", "name"])
